@@ -39,8 +39,8 @@ print.smle<-function(x,...){
   catln(" ")
   catln("Subset:")
   catln("  Model size: ", as.character(x$num_retained))
-  if( !is.null(colnames(x$X))){ catln("  Feature Name: ", paste(colnames(x$X)[x$ID_retained], sep = " ", collapse = ","))}
-  catln("  Feature Index: ", paste(x$ID_retained, sep = "\n", collapse = ","))
+  if( !is.null(x$iteration_data$feature_name)){ catln("  Feature name: ", paste(x$iteration_data$feature_name, sep = "", collapse = ", "))}
+  catln("  Feature index: ", paste(x$ID_retained, sep = "\n", collapse = ", "))
   
 
   ## done
@@ -69,14 +69,30 @@ print.selection<-function(x,...){
   catln(paste("An object of class",class(x)))
   catln(" ")
   catln("Subset:")
-  if(x$vote==TRUE){
-    if( !is.null(colnames(x$X))){ catln("  Feature Name: ", paste(colnames(x$X)[x$ID_voted], sep = " ", collapse = ","))}
-    catln("  Features selected by voting: ",  paste(x$ID_voted, sep = "\n", collapse = ","))
+  if(is.null(x$data)){  
+    if(x$vote==TRUE){
+      if( !is.null(colnames(x$X))){ catln("  Feature name: ", paste(colnames(x$X)[x$ID_voted], sep = "", collapse = ", "))}
+      catln("  Features selected by voting: ",  paste(x$ID_voted, sep = "\n", collapse = ", "))
+    }else{
+      if( !is.null(colnames(x$X))){ catln("  Feature name: ", paste(colnames(x$X)[x$ID_selected], sep = "", collapse = ", "))}  
+      catln("  Feature index: ", paste(x$ID_selected, sep = "\n", collapse = ", "))}
+    ## done
+    return(invisible(x))
+    
   }else{
-    if( !is.null(colnames(x$X))){ catln("  Feature Name: ", paste(colnames(x$X)[x$ID_selected], sep = " ", collapse = ","))}  
-    catln("  Feature Index: ", paste(x$ID_selected, sep = "\n", collapse = ","))}
-  ## done
-  invisible(x)
+    
+    if(x$vote==TRUE){
+      catln("  Feature name: ", paste(names(x$data)[x$ID_voted], sep = "", collapse = ", "))
+      catln("  Features selected by voting: ",  paste(x$ID_voted, sep = "\n", collapse = ", "))
+    }else{
+      catln("  Feature name: ", paste(names(x$data)[x$ID_selected], sep = "", collapse = ", "))  
+      catln("  Feature index: ", paste(x$ID_selected, sep = "\n", collapse = ", "))}
+    ## done
+    return(invisible(x))
+    
+    
+    }
+
   
 }
 
@@ -106,9 +122,9 @@ print.summary.smle <- function(x, ...){
   catln("  Model type: " , x$family)
   catln("  Retained model size: ", x$size)
   
-  if( !is.null(colnames(x$X))){ catln("  Feature Name: ", paste(colnames(x$X)[x$ID_retained], sep = " ", collapse = ","))}
-  catln("  Retained features: ", paste(x$ID_retained, sep = "\n", collapse = ","))
-  catln("  Coefficients estimated by IHT: ", paste(format(x$coef_retained, digits = 3),collapse = ' '))
+  if( !is.null(x$iteration_data$feature_name)){ catln("  Feature name: ", paste(x$iteration_data$feature_name, sep = "", collapse = ", "))}
+  catln("  Retained features: ", paste(x$ID_retained, sep = "\n", collapse = ", "))
+  catln("  Coefficients estimated by IHT: ", paste(format(x$coef_retained, digits = 3,trim = TRUE),collapse = ', '))
   catln("  Number of IHT iteration steps: ",as.character(x$steps))
 
   
@@ -138,11 +154,12 @@ print.summary.selection <- function(x, ...){
   catln("  Dim of features: " , paste(x$DimX,collapse = ' x '))
   catln("  Model type: " , x$family)
   catln("  Selected model size: ", x$size)
-  if( !is.null(colnames(x$X))){ catln("  Feature Name: ", paste(colnames(x$X)[x$ID_selected], sep = " ", collapse = ","))}
-  catln("  Selected features: ", paste(x$ID_selected, sep = "\n", collapse = ","))
+  if( !is.null(x$data)){ catln("  Feature name: ", paste(names(x$data)[x$ID_selected], sep = "", collapse = ", "))
+    }else if( !is.null(colnames(x$X))){ catln("  Feature name: ", paste(colnames(x$X)[x$ID_selected], sep = "", collapse = ", "))}
+  catln("  Selected features: ", paste(x$ID_selected, sep = "\n", collapse = ", "))
   catln("  Selection criterion: ", x$criterion)
   if(x$criterion=='ebic'){  catln("  Gamma for ebic: ", as.character(x$gamma_ebic))}
-  if(x$vote==TRUE){  catln("  Features selected by voting: ",  paste(x$ID_voted, sep = "\n", collapse = ","))}
+  if(x$vote==TRUE){  catln("  Features selected by voting: ",  paste(x$ID_voted, sep = "\n", collapse = ", "))}
   
   invisible(x)
 }
@@ -172,9 +189,9 @@ print.sdata<-function(x,...){
   catln(" Dim of features: " , paste(dim(x$X),collapse = ' x '))
   catln(" Correlation: " , x$correlation)
   if(x$correlation != "independent"){catln(" Rho: ", x$rho)}
-  catln(" Index of Causal Features: " , paste(x$subset_true,collapse = ','))
-  if(x$ctg ==T){catln(" Design matrix concludes categorical features" )}
-  catln(" Model Type: ", x$family)
+  catln(" Index of causal features: " , paste(x$subset_true,collapse = ', '))
+  if(x$categorical ==T){catln(" Contain categorical features: TRUE" )}
+  catln(" Model type: ", x$family)
 
   ## done
   invisible(x)
